@@ -5,25 +5,41 @@ from pipenv import environments
 from pipenv.__version__ import __version__
 from pipenv._compat import fix_utf8
 from pipenv.cli.options import (
-    CONTEXT_SETTINGS, PipenvGroup, code_option, common_options, deploy_option,
-    general_options, install_options, lock_options, pass_state,
-    pypi_mirror_option, python_option, site_packages_option, skip_lock_option,
-    sync_options, system_option, three_option, uninstall_options, verbose_option
+    CONTEXT_SETTINGS,
+    PipenvGroup,
+    common_options,
+    deploy_option,
+    general_options,
+    install_options,
+    lock_options,
+    pass_state,
+    pypi_mirror_option,
+    python_option,
+    site_packages_option,
+    skip_lock_option,
+    sync_options,
+    system_option,
+    three_option,
+    uninstall_options,
+    verbose_option,
 )
 from pipenv.exceptions import PipenvOptionsError
 from pipenv.patched import crayons
-from pipenv.utils import subprocess_run
+from pipenv.utils.processes import subprocess_run
 from pipenv.vendor.click import (
-    Choice, argument, echo, edit, group, option, pass_context, secho, types,
-    version_option
+    Choice,
+    argument,
+    echo,
+    edit,
+    group,
+    option,
+    pass_context,
+    secho,
+    version_option,
 )
 
-
 subcommand_context = CONTEXT_SETTINGS.copy()
-subcommand_context.update({
-    "ignore_unknown_options": True,
-    "allow_extra_args": True
-})
+subcommand_context.update({"ignore_unknown_options": True, "allow_extra_args": True})
 subcommand_context_no_interspersion = subcommand_context.copy()
 subcommand_context_no_interspersion["allow_interspersed_args"] = False
 
@@ -31,8 +47,12 @@ subcommand_context_no_interspersion["allow_interspersed_args"] = False
 @group(cls=PipenvGroup, invoke_without_command=True, context_settings=CONTEXT_SETTINGS)
 @option("--where", is_flag=True, default=False, help="Output project home information.")
 @option("--venv", is_flag=True, default=False, help="Output virtualenv information.")
-@option("--py", is_flag=True, default=False, help="Output Python interpreter information.")
-@option("--envs", is_flag=True, default=False, help="Output Environment Variable options.")
+@option(
+    "--py", is_flag=True, default=False, help="Output Python interpreter information."
+)
+@option(
+    "--envs", is_flag=True, default=False, help="Output Environment Variable options."
+)
 @option("--rm", is_flag=True, default=False, help="Remove the virtualenv.")
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
 @option("--man", is_flag=True, default=False, help="Display manpage.")
@@ -58,13 +78,20 @@ def cli(
     support=None,
     help=False,
     site_packages=None,
-    **kwargs
+    **kwargs,
 ):
+    from pipenv.utils.spinner import create_spinner
+
     from ..core import (
-        cleanup_virtualenv, do_clear, do_py, do_where, ensure_project,
-        format_help, system_which, warn_in_virtualenv
+        cleanup_virtualenv,
+        do_clear,
+        do_py,
+        do_where,
+        ensure_project,
+        format_help,
+        system_which,
+        warn_in_virtualenv,
     )
-    from ..utils import create_spinner
 
     if man:
         if system_which("man"):
@@ -72,7 +99,12 @@ def cli(
             os.execle(system_which("man"), "man", path, os.environ)
             return 0
         else:
-            secho("man does not appear to be available on your system.", fg="yellow", bold=True, err=True)
+            secho(
+                "man does not appear to be available on your system.",
+                fg="yellow",
+                bold=True,
+                err=True,
+            )
             return 1
     if envs:
         echo("The following environment variables can be set, to do various things:\n")
@@ -114,7 +146,7 @@ def cli(
                     "{}({}){}".format(
                         crayons.red("No virtualenv has been created for this project"),
                         crayons.normal(state.project.project_directory, bold=True),
-                        crayons.red(" yet!")
+                        crayons.red(" yet!"),
                     ),
                     err=True,
                 )
@@ -156,7 +188,7 @@ def cli(
                     err=True,
                 )
                 ctx.abort()
-    # --two / --three was passed...
+    # --python or --three was passed...
     if (state.python or state.three is not None) or state.site_packages:
         ensure_project(
             state.project,
@@ -178,16 +210,12 @@ def cli(
     context_settings=subcommand_context,
 )
 @system_option
-@code_option
 @deploy_option
 @site_packages_option
 @skip_lock_option
 @install_options
 @pass_state
-def install(
-    state,
-    **kwargs
-):
+def install(state, **kwargs):
     """Installs provided packages and adds them to Pipfile, or (if no packages are given), installs all packages from Pipfile."""
     from ..core import do_install
 
@@ -204,21 +232,19 @@ def install(
         requirementstxt=state.installstate.requirementstxt,
         sequential=state.installstate.sequential,
         pre=state.installstate.pre,
-        code=state.installstate.code,
         deploy=state.installstate.deploy,
         keep_outdated=state.installstate.keep_outdated,
         selective_upgrade=state.installstate.selective_upgrade,
         index_url=state.index,
-        extra_index_url=state.extra_index_urls,
         packages=state.installstate.packages,
         editable_packages=state.installstate.editables,
-        site_packages=state.site_packages
+        site_packages=state.site_packages,
     )
 
 
 @cli.command(
     short_help="Uninstalls a provided package and removes it from Pipfile.",
-    context_settings=subcommand_context
+    context_settings=subcommand_context,
 )
 @option(
     "--all-dev",
@@ -235,15 +261,10 @@ def install(
 @uninstall_options
 @pass_state
 @pass_context
-def uninstall(
-    ctx,
-    state,
-    all_dev=False,
-    all=False,
-    **kwargs
-):
+def uninstall(ctx, state, all_dev=False, all=False, **kwargs):
     """Uninstalls a provided package and removes it from Pipfile."""
     from ..core import do_uninstall
+
     retcode = do_uninstall(
         state.project,
         packages=state.installstate.packages,
@@ -256,7 +277,7 @@ def uninstall(
         all=all,
         keep_outdated=state.installstate.keep_outdated,
         pypi_mirror=state.pypi_mirror,
-        ctx=ctx
+        ctx=ctx,
     )
     if retcode:
         sys.exit(retcode)
@@ -282,11 +303,7 @@ LOCK_DEV_NOTE = """\
 @lock_options
 @pass_state
 @pass_context
-def lock(
-    ctx,
-    state,
-    **kwargs
-):
+def lock(ctx, state, **kwargs):
     """Generates Pipfile.lock."""
     from ..core import do_init, do_lock, ensure_project
 
@@ -294,14 +311,27 @@ def lock(
     # Note that we don't pass clear on to ensure_project as it is also
     # handled in do_lock
     ensure_project(
-        state.project, three=state.three, python=state.python, pypi_mirror=state.pypi_mirror,
-        warn=(not state.quiet), site_packages=state.site_packages,
+        state.project,
+        three=state.three,
+        python=state.python,
+        pypi_mirror=state.pypi_mirror,
+        warn=(not state.quiet),
+        site_packages=state.site_packages,
     )
     emit_requirements = state.lockoptions.emit_requirements
     dev = state.installstate.dev
     dev_only = state.lockoptions.dev_only
     pre = state.installstate.pre
     if emit_requirements:
+        secho(
+            "Warning: The lock flag -r/--requirements will be deprecated in a future version\n"
+            "of pipenv in favor of the new requirements command. For more info see\n"
+            "https://pipenv.pypa.io/en/latest/advanced/#generating-a-requirements-txt\n"
+            "NOTE: the requirements command parses Pipfile.lock directly without performing any\n"
+            "locking operations. Updating packages should be done by running pipenv lock.",
+            fg="yellow",
+            err=True,
+        )
         # Emit requirements file header (unless turned off with --no-header)
         if state.lockoptions.emit_requirements_header:
             header_options = ["--requirements"]
@@ -327,7 +357,7 @@ def lock(
         raise PipenvOptionsError(
             "--dev-only",
             "--dev-only is only permitted in combination with --requirements. "
-            "Aborting."
+            "Aborting.",
         )
     do_lock(
         state.project,
@@ -349,7 +379,7 @@ def lock(
     is_flag=True,
     default=False,
     help="Run in shell in fancy mode. Make sure the shell have no path manipulating"
-         " scripts. Run $pipenv shell for issues with compatibility mode.",
+    " scripts. Run $pipenv shell for issues with compatibility mode.",
 )
 @option(
     "--anyway",
@@ -411,29 +441,29 @@ def shell(
 def run(state, command, args):
     """Spawns a command installed into the virtualenv."""
     from ..core import do_run
+
     do_run(
-        state.project, command=command, args=args, three=state.three, python=state.python, pypi_mirror=state.pypi_mirror
+        state.project,
+        command=command,
+        args=args,
+        three=state.three,
+        python=state.python,
+        pypi_mirror=state.pypi_mirror,
+        quiet=state.quiet,
     )
 
 
 @cli.command(
     short_help="Checks for PyUp Safety security vulnerabilities and against"
-               " PEP 508 markers provided in Pipfile.",
-    context_settings=subcommand_context
-)
-@option(
-    "--unused",
-    nargs=1,
-    default="",
-    type=types.STRING,
-    help="Given a code path, show potentially unused dependencies.",
+    " PEP 508 markers provided in Pipfile.",
+    context_settings=subcommand_context,
 )
 @option(
     "--db",
     nargs=1,
-    default=lambda: os.environ.get('PIPENV_SAFETY_DB'),
+    default=lambda: os.environ.get("PIPENV_SAFETY_DB"),
     help="Path to a local PyUp Safety vulnerabilities database."
-         " Default: ENV PIPENV_SAFETY_DB or None.",
+    " Default: ENV PIPENV_SAFETY_DB or None.",
 )
 @option(
     "--ignore",
@@ -450,29 +480,24 @@ def run(state, command, args):
 @option(
     "--key",
     help="Safety API key from PyUp.io for scanning dependencies against a live"
-         " vulnerabilities database. Leave blank for scanning against a"
-         " database that only updates once a month.",
+    " vulnerabilities database. Leave blank for scanning against a"
+    " database that only updates once a month.",
 )
 @option(
-    "--quiet",
-    is_flag=True,
-    help="Quiet standard output, except vulnerability report."
+    "--quiet", is_flag=True, help="Quiet standard output, except vulnerability report."
 )
 @common_options
 @system_option
-@argument("args", nargs=-1)
 @pass_state
 def check(
     state,
-    unused=False,
     db=None,
     style=False,
     ignore=None,
     output="default",
     key=None,
     quiet=False,
-    args=None,
-    **kwargs
+    **kwargs,
 ):
     """Checks for PyUp Safety security vulnerabilities and against PEP 508 markers provided in Pipfile."""
     from ..core import do_check
@@ -482,44 +507,44 @@ def check(
         three=state.three,
         python=state.python,
         system=state.system,
-        unused=unused,
         db=db,
         ignore=ignore,
         output=output,
         key=key,
         quiet=quiet,
-        args=args,
         pypi_mirror=state.pypi_mirror,
     )
 
 
 @cli.command(short_help="Runs lock, then sync.", context_settings=CONTEXT_SETTINGS)
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
-@option(
-    "--outdated", is_flag=True, default=False, help="List out-of-date dependencies."
-)
+@option("--outdated", is_flag=True, default=False, help="List out-of-date dependencies.")
 @option("--dry-run", is_flag=True, default=None, help="List out-of-date dependencies.")
 @install_options
 @pass_state
 @pass_context
-def update(
-    ctx,
-    state,
-    bare=False,
-    dry_run=None,
-    outdated=False,
-    **kwargs
-):
+def update(ctx, state, bare=False, dry_run=None, outdated=False, **kwargs):
     """Runs lock, then sync."""
     from ..core import do_lock, do_outdated, do_sync, ensure_project
+
     ensure_project(
-        state.project, three=state.three, python=state.python, pypi_mirror=state.pypi_mirror,
-        warn=(not state.quiet), site_packages=state.site_packages, clear=state.clear
+        state.project,
+        three=state.three,
+        python=state.python,
+        pypi_mirror=state.pypi_mirror,
+        warn=(not state.quiet),
+        site_packages=state.site_packages,
+        clear=state.clear,
     )
     if not outdated:
         outdated = bool(dry_run)
     if outdated:
-        do_outdated(state.project, clear=state.clear, pre=state.installstate.pre, pypi_mirror=state.pypi_mirror)
+        do_outdated(
+            state.project,
+            clear=state.clear,
+            pre=state.installstate.pre,
+            pypi_mirror=state.pypi_mirror,
+        )
     packages = [p for p in state.installstate.packages if p]
     editable = [p for p in state.installstate.editables if p]
     if not packages:
@@ -570,7 +595,7 @@ def update(
 
 @cli.command(
     short_help="Displays currently-installed dependency graph information.",
-    context_settings=CONTEXT_SETTINGS
+    context_settings=CONTEXT_SETTINGS,
 )
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
 @option("--json", is_flag=True, default=False, help="Output JSON.")
@@ -585,8 +610,9 @@ def graph(state, bare=False, json=False, json_tree=False, reverse=False):
 
 
 @cli.command(
-    short_help="View a given module in your editor.", name="open",
-    context_settings=CONTEXT_SETTINGS
+    short_help="View a given module in your editor.",
+    name="open",
+    context_settings=CONTEXT_SETTINGS,
 )
 @common_options
 @argument("module", nargs=1)
@@ -603,11 +629,18 @@ def run_open(state, module, *args, **kwargs):
 
     # Ensure that virtualenv is available.
     ensure_project(
-        state.project, three=state.three, python=state.python,
-        validate=False, pypi_mirror=state.pypi_mirror,
+        state.project,
+        three=state.three,
+        python=state.python,
+        validate=False,
+        pypi_mirror=state.pypi_mirror,
     )
     c = subprocess_run(
-        [state.project._which("python"), "-c", "import {0}; print({0}.__file__)".format(module)]
+        [
+            state.project._which("python"),
+            "-c",
+            "import {0}; print({0}.__file__)".format(module),
+        ]
     )
     if c.returncode:
         echo(crayons.red("Module not found!"))
@@ -624,21 +657,14 @@ def run_open(state, module, *args, **kwargs):
 
 @cli.command(
     short_help="Installs all packages specified in Pipfile.lock.",
-    context_settings=CONTEXT_SETTINGS
+    context_settings=CONTEXT_SETTINGS,
 )
 @system_option
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
 @sync_options
 @pass_state
 @pass_context
-def sync(
-    ctx,
-    state,
-    bare=False,
-    user=False,
-    unused=False,
-    **kwargs
-):
+def sync(ctx, state, bare=False, user=False, unused=False, **kwargs):
     """Installs all packages specified in Pipfile.lock."""
     from ..core import do_sync
 
@@ -654,7 +680,7 @@ def sync(
         unused=unused,
         sequential=state.installstate.sequential,
         pypi_mirror=state.pypi_mirror,
-        system=state.system
+        system=state.system,
     )
     if retcode:
         ctx.abort()
@@ -662,7 +688,7 @@ def sync(
 
 @cli.command(
     short_help="Uninstalls all packages not specified in Pipfile.lock.",
-    context_settings=CONTEXT_SETTINGS
+    context_settings=CONTEXT_SETTINGS,
 )
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
 @option("--dry-run", is_flag=True, default=False, help="Just output unneeded packages.")
@@ -673,8 +699,14 @@ def sync(
 def clean(state, dry_run=False, bare=False, user=False):
     """Uninstalls all packages not specified in Pipfile.lock."""
     from ..core import do_clean
-    do_clean(state.project, three=state.three, python=state.python, dry_run=dry_run,
-             system=state.system)
+
+    do_clean(
+        state.project,
+        three=state.three,
+        python=state.python,
+        dry_run=dry_run,
+        system=state.system,
+    )
 
 
 @cli.command(
@@ -688,7 +720,7 @@ def scripts(state):
     if not state.project.pipfile_exists:
         echo("No Pipfile present at project home.", err=True)
         sys.exit(1)
-    scripts = state.project.parsed_pipfile.get('scripts', {})
+    scripts = state.project.parsed_pipfile.get("scripts", {})
     first_column_width = max(len(word) for word in ["Command"] + list(scripts))
     second_column_width = max(len(word) for word in ["Script"] + list(scripts.values()))
     lines = ["{0:<{width}}  Script".format("Command", width=first_column_width)]
@@ -700,5 +732,69 @@ def scripts(state):
     echo("\n".join(fix_utf8(line) for line in lines))
 
 
+@cli.command(
+    short_help="Verify the hash in Pipfile.lock is up-to-date.",
+    context_settings=CONTEXT_SETTINGS,
+)
+@pass_state
+def verify(state):
+    """Verify the hash in Pipfile.lock is up-to-date."""
+    if not state.project.pipfile_exists:
+        echo("No Pipfile present at project home.", err=True)
+        sys.exit(1)
+    if state.project.get_lockfile_hash() != state.project.calculate_pipfile_hash():
+        echo(
+            "Pipfile.lock is out-of-date. Run {} to update.".format(
+                crayons.yellow("$ pipenv lock", bold=True)
+            ),
+            err=True,
+        )
+        sys.exit(1)
+    echo(crayons.green("Pipfile.lock is up-to-date."))
+    sys.exit(0)
+
+
+@cli.command(
+    short_help="Generate a requirements.txt from Pipfile.lock.",
+    context_settings=CONTEXT_SETTINGS,
+)
+@option("--dev", is_flag=True, default=False, help="Also add development requirements.")
+@option(
+    "--dev-only", is_flag=True, default=False, help="Only add development requirements."
+)
+@option("--hash", is_flag=True, default=False, help="Add package hashes.")
+@pass_state
+def requirements(state, dev=False, dev_only=False, hash=False):
+
+    from pipenv.utils.dependencies import convert_deps_to_pip
+
+    lockfile = state.project.lockfile_content
+
+    for i, package_index in enumerate(lockfile["_meta"]["sources"]):
+        prefix = "-i" if i == 0 else "--extra-index-url"
+        echo(" ".join([prefix, package_index["url"]]))
+
+    deps = {}
+
+    if not dev_only:
+        deps.update(lockfile["default"])
+    if dev or dev_only:
+        deps.update(lockfile["develop"])
+
+    pip_deps = convert_deps_to_pip(
+        deps,
+        project=None,
+        r=False,
+        include_index=False,
+        include_hashes=hash,
+        include_markers=False,
+    )
+
+    for d in pip_deps:
+        echo(d)
+
+    sys.exit(0)
+
+
 if __name__ == "__main__":
     cli()
