@@ -1,9 +1,6 @@
-from typing import Optional
-
-from pipenv.patched.notpip._vendor.pkg_resources import Distribution
-
 from pipenv.patched.notpip._internal.distributions.base import AbstractDistribution
 from pipenv.patched.notpip._internal.index.package_finder import PackageFinder
+from pipenv.patched.notpip._internal.metadata import BaseDistribution
 
 
 class InstalledDistribution(AbstractDistribution):
@@ -13,7 +10,8 @@ class InstalledDistribution(AbstractDistribution):
     been computed.
     """
 
-    def get_pkg_resources_distribution(self) -> Optional[Distribution]:
+    def get_metadata_distribution(self) -> BaseDistribution:
+        assert self.req.satisfied_by is not None, "not actually installed"
         return self.req.satisfied_by
 
     def prepare_distribution_metadata(
