@@ -1,17 +1,22 @@
 # -*- coding: utf-8 -*-
-import requests
-from packaging.specifiers import SpecifierSet
-from .errors import DatabaseFetchError, InvalidKeyError, DatabaseFileNotFoundError
-from .constants import OPEN_MIRRORS, API_MIRRORS, REQUEST_TIMEOUT, CACHE_VALID_SECONDS, CACHE_FILE
-from collections import namedtuple
-import os
-import json
-import time
 import errno
+import json
+import os
+import time
+from collections import namedtuple
+
+import pipenv.vendor.requests as requests
+from pipenv.vendor.packaging.specifiers import SpecifierSet
+
+from .constants import (API_MIRRORS, CACHE_FILE, CACHE_LICENSES_VALID_SECONDS,
+                        CACHE_VALID_SECONDS, OPEN_MIRRORS, REQUEST_TIMEOUT)
+from .errors import (DatabaseFetchError, DatabaseFileNotFoundError,
+                     InvalidKeyError, TooManyRequestsError)
+from .util import RequirementFile
 
 
 class Vulnerability(namedtuple("Vulnerability",
-                               ["name", "spec", "version", "advisory", "vuln_id"])):
+                               ["name", "spec", "version", "advisory", "vuln_id", "cvssv2", "cvssv3"])):
     pass
 
 
@@ -22,7 +27,13 @@ def get_from_cache(db_name):
                 data = json.loads(f.read())
                 if db_name in data:
                     if "cached_at" in data[db_name]:
-                        if data[db_name]["cached_at"] + CACHE_VALID_SECONDS > time.time():
+                        if 'licenses.json' in db_name:
+                            # Getting the specific cache time for the licenses db.
+                            cache_valid_seconds = CACHE_LICENSES_VALID_SECONDS
+                        else:
+                            cache_valid_seconds = CACHE_VALID_SECONDS
+
+                        if data[db_name]["cached_at"] + cache_valid_seconds > time.time():
                             return data[db_name]["db"]
             except json.JSONDecodeError:
                 pass
@@ -84,6 +95,8 @@ def fetch_database_url(mirror, db_name, key, cached, proxy):
         return data
     elif r.status_code == 403:
         raise InvalidKeyError()
+    elif r.status_code == 429:
+        raise TooManyRequestsError()
 
 
 def fetch_database_file(path, db_name):
@@ -127,6 +140,10 @@ def check(packages, key, db_mirror, cached, ignore_ids, proxy):
     vulnerable_packages = frozenset(db.keys())
     vulnerable = []
     for pkg in packages:
+        # Ignore recursive files not resolved
+        if isinstance(pkg, RequirementFile):
+            continue
+
         # normalize the package name, the safety-db is converting underscores to dashes and uses
         # lowercase
         name = pkg.key.replace("_", "-").lower()
@@ -140,14 +157,20 @@ def check(packages, key, db_mirror, cached, ignore_ids, proxy):
                         db_full = fetch_database(full=True, key=key, db=db_mirror, cached=cached, proxy=proxy)
                     for data in get_vulnerabilities(pkg=name, spec=specifier, db=db_full):
                         vuln_id = data.get("id").replace("pyup.io-", "")
+                        cve_id = data.get("cve")
+                        if cve_id:
+                            cve_id = cve_id.split(",")[0].strip()
                         if vuln_id and vuln_id not in ignore_ids:
+                            cve_meta = db_full.get("$meta", {}).get("cve", {}).get(cve_id, {})
                             vulnerable.append(
                                 Vulnerability(
                                     name=name,
                                     spec=specifier,
                                     version=pkg.version,
                                     advisory=data.get("advisory"),
-                                    vuln_id=vuln_id
+                                    vuln_id=vuln_id,
+                                    cvssv2=cve_meta.get("cvssv2", None),
+                                    cvssv3=cve_meta.get("cvssv3", None),
                                 )
                             )
     return vulnerable
@@ -162,8 +185,33 @@ def review(vulnerabilities):
             "version": vuln[2],
             "advisory": vuln[3],
             "vuln_id": vuln[4],
+            "cvssv2": None,
+            "cvssv3": None
         }
         vulnerable.append(
             Vulnerability(**current_vuln)
         )
     return vulnerable
+
+
+def get_licenses(key, db_mirror, cached, proxy):
+    key = key if key else os.environ.get("SAFETY_API_KEY", False)
+
+    if not key and not db_mirror:
+        raise InvalidKeyError("The API-KEY was not provided.")
+    if db_mirror:
+        mirrors = [db_mirror]
+    else:
+        mirrors = API_MIRRORS
+
+    db_name = "licenses.json"
+
+    for mirror in mirrors:
+        # mirror can either be a local path or a URL
+        if mirror.startswith("http://") or mirror.startswith("https://"):
+            licenses = fetch_database_url(mirror, db_name=db_name, key=key, cached=cached, proxy=proxy)
+        else:
+            licenses = fetch_database_file(mirror, db_name=db_name)
+        if licenses:
+            return licenses
+    raise DatabaseFetchError()
