@@ -29,7 +29,6 @@ from pipenv.environments import (
 from pipenv.utils.dependencies import (
     get_canonical_names,
     is_editable,
-    is_installable_file,
     is_star,
     python_version,
 )
@@ -155,58 +154,8 @@ class Project:
         return os.sep.join([self._original_dir, p])
 
     def _build_package_list(self, package_section):
-        """Returns a list of packages for pip-tools to consume."""
-        from pipenv.vendor.requirementslib.utils import is_vcs
-
-        ps = {}
-        # TODO: Separate the logic for showing packages from the filters for supplying pip-tools
-        for k, v in self.parsed_pipfile.get(package_section, {}).items():
-            # Skip editable VCS deps.
-            if hasattr(v, "keys"):
-                # When a vcs url is gven without editable it only appears as a key
-                # Eliminate any vcs, path, or url entries which are not editable
-                # Since pip-tools can't do deep resolution on them, even setuptools-installable ones
-                if (
-                    is_vcs(v)
-                    or is_vcs(k)
-                    or (is_installable_file(k) or is_installable_file(v))
-                    or any(
-                        (
-                            prefix in v
-                            and (os.path.isfile(v[prefix]) or is_valid_url(v[prefix]))
-                        )
-                        for prefix in ["path", "file"]
-                    )
-                ):
-                    # If they are editable, do resolve them
-                    if "editable" not in v:
-                        # allow wheels to be passed through
-                        if not (
-                            hasattr(v, "keys")
-                            and v.get("path", v.get("file", "")).endswith(".whl")
-                        ):
-                            continue
-                        ps.update({k: v})
-
-                    else:
-                        ps.update({k: v})
-                else:
-                    ps.update({k: v})
-            else:
-                # Since these entries have no attributes we know they are not editable
-                # So we can safely exclude things that need to be editable in order to be resolved
-                # First exclude anything that is a vcs entry either in the key or value
-                if not (
-                    any(is_vcs(i) for i in [k, v])
-                    # Then exclude any installable files that are not directories
-                    # Because pip-tools can resolve setup.py for example
-                    or any(is_installable_file(i) for i in [k, v])
-                    # Then exclude any URLs because they need to be editable also
-                    # Things that are excluded can only be 'shallow resolved'
-                    or any(is_valid_url(i) for i in [k, v])
-                ):
-                    ps.update({k: v})
-        return ps
+        """Returns the list of packages from the Project's Pipfile."""
+        return self.parsed_pipfile.get(package_section, {})
 
     @property
     def name(self):
@@ -693,12 +642,12 @@ class Project:
 
     @property
     def vcs_packages(self):
-        """Returns a list of VCS packages, for not pip-tools to consume."""
+        """Returns a list of VCS packages."""
         return self._get_vcs_packages(dev=False)
 
     @property
     def vcs_dev_packages(self):
-        """Returns a list of VCS packages, for not pip-tools to consume."""
+        """Returns a list of VCS packages."""
         return self._get_vcs_packages(dev=True)
 
     @property
@@ -710,12 +659,12 @@ class Project:
 
     @property
     def packages(self):
-        """Returns a list of packages, for pip-tools to consume."""
+        """Returns a list of packages."""
         return self._build_package_list("packages")
 
     @property
     def dev_packages(self):
-        """Returns a list of dev-packages, for pip-tools to consume."""
+        """Returns a list of dev-packages."""
         return self._build_package_list("dev-packages")
 
     @property
