@@ -5,9 +5,10 @@ import re
 import signal
 import subprocess
 import sys
+from pathlib import Path
 
 from pipenv.vendor import shellingham
-from pipenv.vendor.vistir.compat import Path, get_terminal_size
+from pipenv.vendor.vistir.compat import get_terminal_size
 from pipenv.vendor.vistir.contextmanagers import temp_environ
 
 ShellDetectionFailure = shellingham.ShellDetectionFailure
