@@ -131,7 +131,6 @@ def translate_markers(pipfile_entry):
     if not isinstance(pipfile_entry, Mapping):
         raise TypeError("Entry is not a pipfile formatted mapping.")
     from pipenv.vendor.packaging.markers import default_environment
-    from pipenv.vendor.vistir.misc import dedup
 
     allowed_marker_keys = ["markers"] + list(default_environment().keys())
     provided_keys = list(pipfile_entry.keys()) if hasattr(pipfile_entry, "keys") else []
@@ -153,7 +152,8 @@ def translate_markers(pipfile_entry):
         new_pipfile["markers"] = str(
             Marker(
                 " or ".join(
-                    f"{s}" if " and " in s else s for s in sorted(dedup(marker_set))
+                    f"{s}" if " and " in s else s
+                    for s in sorted(dict.fromkeys(marker_set))
                 )
             )
         ).replace('"', "'")
@@ -243,7 +243,14 @@ def is_pinned_requirement(ireq):
     return spec.operator in {"==", "==="} and not spec.version.endswith(".*")
 
 
-def convert_deps_to_pip(deps, project=None, r=True, include_index=True):
+def convert_deps_to_pip(
+    deps,
+    project=None,
+    r=True,
+    include_index=True,
+    include_hashes=True,
+    include_markers=True,
+):
     """ "Converts a Pipfile-formatted dependency to a pip-formatted one."""
     from pipenv.vendor.requirementslib.models.requirements import Requirement
 
@@ -255,7 +262,12 @@ def convert_deps_to_pip(deps, project=None, r=True, include_index=True):
         new_dep = Requirement.from_pipfile(dep_name, dep)
         if new_dep.index:
             include_index = True
-        req = new_dep.as_line(sources=indexes if include_index else None).strip()
+        sources = indexes if include_index else None
+        req = new_dep.as_line(
+            sources=sources,
+            include_hashes=include_hashes,
+            include_markers=include_markers,
+        ).strip()
         dependencies.append(req)
     if not r:
         return dependencies
