@@ -518,7 +518,7 @@ class Resolver:
             pip_options, _ = self.pip_command.parser.parse_args(self.pip_args)
             pip_options.cache_dir = self.project.s.PIPENV_CACHE_DIR
             pip_options.no_python_version_warning = True
-            pip_options.no_input = True
+            pip_options.no_input = self.project.settings.get("disable_pip_input", True)
             pip_options.progress_bar = "off"
             pip_options.ignore_requires_python = True
             pip_options.pre = self.pre or self.project.settings.get(
@@ -735,6 +735,8 @@ class Resolver:
         link = ireq.link  # Handle VCS and file links first
         if link and (link.is_vcs or (link.is_file and link.is_existing_dir())):
             return set()
+        if link and ireq.original_link:
+            return {self._get_hash_from_link(ireq.original_link)}
 
         if not is_pinned_requirement(ireq):
             return set()
@@ -760,8 +762,6 @@ class Resolver:
             }
         if link:
             return {self._get_hash_from_link(link)}
-        if ireq.original_link:
-            return {self._get_hash_from_link(ireq.original_link)}
         return set()
 
     def resolve_hashes(self):
@@ -943,11 +943,11 @@ def venv_resolve_deps(
     """
 
     import json
+    import tempfile
 
     from pipenv import resolver
     from pipenv._compat import decode_for_output
-    from pipenv.vendor.vistir.compat import JSONDecodeError, NamedTemporaryFile, Path
-    from pipenv.vendor.vistir.misc import fs_str
+    from pipenv.vendor.vistir.compat import Path
 
     results = []
     pipfile_section = "dev-packages" if dev else "packages"
@@ -976,23 +976,25 @@ def venv_resolve_deps(
         cmd.append("--system")
     if dev:
         cmd.append("--dev")
-    target_file = NamedTemporaryFile(prefix="resolver", suffix=".json", delete=False)
+    target_file = tempfile.NamedTemporaryFile(
+        prefix="resolver", suffix=".json", delete=False
+    )
     target_file.close()
     cmd.extend(["--write", make_posix(target_file.name)])
     with temp_environ():
-        os.environ.update({fs_str(k): fs_str(val) for k, val in os.environ.items()})
+        os.environ.update({k: str(val) for k, val in os.environ.items()})
         if pypi_mirror:
             os.environ["PIPENV_PYPI_MIRROR"] = str(pypi_mirror)
         os.environ["PIPENV_VERBOSITY"] = str(project.s.PIPENV_VERBOSITY)
-        os.environ["PIPENV_REQ_DIR"] = fs_str(req_dir)
-        os.environ["PIP_NO_INPUT"] = fs_str("1")
+        os.environ["PIPENV_REQ_DIR"] = req_dir
+        os.environ["PIP_NO_INPUT"] = "1"
         pipenv_site_dir = get_pipenv_sitedir()
         if pipenv_site_dir is not None:
             os.environ["PIPENV_SITE_DIR"] = pipenv_site_dir
         else:
             os.environ.pop("PIPENV_SITE_DIR", None)
         if keep_outdated:
-            os.environ["PIPENV_KEEP_OUTDATED"] = fs_str("1")
+            os.environ["PIPENV_KEEP_OUTDATED"] = "1"
         with create_spinner(
             text=decode_for_output("Locking..."), setting=project.s
         ) as sp:
@@ -1020,7 +1022,7 @@ def venv_resolve_deps(
     try:
         with open(target_file.name) as fh:
             results = json.load(fh)
-    except (IndexError, JSONDecodeError):
+    except (IndexError, json.JSONDecodeError):
         click_echo(c.stdout.strip(), err=True)
         click_echo(c.stderr.strip(), err=True)
         if os.path.exists(target_file.name):
@@ -1045,7 +1047,7 @@ def resolve_deps(
     req_dir=None,
 ):
     """Given a list of dependencies, return a resolved list of dependencies,
-    using pip-tools -- and their hashes, using the warehouse API / pip.
+    and their hashes, using the warehouse API / pip.
     """
     index_lookup = {}
     markers_lookup = {}
